#
# PaiMei
# Copyright (C) 2006 Pedram Amini <pedram.amini@gmail.com>
#
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

'''
@author:       Pedram Amini.  Fuzzing Modifications by Jared DeMott.
@license:      GNU General Public License 2.0 or later
@contact:      pedram.amini@gmail.com
@organization: www.openrce.org
'''

import wx
import copy
import os
import time

from wx.lib.mixins.listctrl import ColumnSorterMixin
from wx.lib.mixins.listctrl import ListCtrlAutoWidthMixin

from pydbg import *
import utils

class ProcessListCtrl (wx.ListCtrl, ListCtrlAutoWidthMixin):
    '''
    Our custom list control containing the hits for the current target/tag.
    '''

    FUNCTIONS    = utils.process_stalker.FUNCTIONS
    BASIC_BLOCKS = utils.process_stalker.BASIC_BLOCKS

    def __init__(self, parent, id, pos=None, size=None, style=None, top=None):
        wx.ListCtrl.__init__(self, parent, id, style=wx.LC_REPORT | wx.SIMPLE_BORDER | wx.LC_HRULES )
        self.top                 = top
        self.restore_breakpoints = False
        self.selected_pid        = 0
        self.selected_proc       = None
        self.process_stalker     = None

        ListCtrlAutoWidthMixin.__init__(self)

        self.InsertColumn(0, "PID")
        self.InsertColumn(1, "Process")

    ####################################################################################################################
    def on_attach_detach (self, event):
        '''
        This is the meat and potatoes. Grab the coverage depth, attach to the selected process, load the appropriate
        modules ... etc etc.
        '''

        ###
        ### staying, detaching, or killing
        ###

        after_stalk = self.top.after_stalk.GetSelection()
        on_crash = self.top.on_crash.GetSelection()

        if self.process_stalker and self.top.crashed == 1 and on_crash == 1:
            self.process_stalker.export = False
            self.process_stalker.detach = False
            self.process_stalker        = None
            self.top.attach_detach.SetLabel("Start Stalking")
            return
        elif self.process_stalker and after_stalk == 1:
            self.process_stalker.export = True
            self.process_stalker.detach = True
            self.process_stalker        = None
            self.top.attach_detach.SetLabel("Start Stalking")
            return
        elif self.process_stalker and after_stalk == 2:
            try:
                dbg.terminate_process()
            except:
                self.top.err("failed to kill process, trying to detach")
                self.process_stalker.export = True
                self.process_stalker.detach = True
            self.process_stalker        = None
            self.top.attach_detach.SetLabel("Start Stalking")
            return
        elif self.process_stalker:
            self.process_stalker.export = True
            self.process_stalker.detach = False
            self.top.attach_detach.SetLabel("Still Stalking")
            return

        ###
        ### attaching / loading ...
        ###

        # sanity checking.
        if not len(self.top.pida_modules):
            self.top.err("You must load at least one PIDA file.")
            return

        if not self.top.stalk_tag:
            self.top.err("You must select a tag for code coverage data storage.")
            return

        # pull the value from the load target control, if anything is specified.
        start = self.top.start.GetValue().rstrip(" ").lstrip(" ")
        #error = self.top.error.GetValue().rstrip(" ").lstrip(" ")
        attach = self.top.attach.GetValue().rstrip(" ").lstrip(" ")
    
        self.top.hits_db = self.top.save_hits.GetSelection()

        if not self.selected_pid and not self.top.attach:
            self.top.err("You must select a target process or executable to stalk.")
            return

        for module in self.top.pida_modules:
            self.top.msg("Stalking module %s" % module)

        # create a new debugger instance for this stalk.
        if hasattr(self.top.main_frame.pydbg, "port"):
            dbg = pydbg_client(self.top.main_frame.pydbg.host, self.top.main_frame.pydbg.port)
        else:
            dbg = pydbg()

        #filled target app control takes precedence over manually selected process
        #use_if_running = self.top.use_if_running.GetValue()
        process_found = False
        started = False
        
        self.top.s_wait = float(self.top.start_wait.GetLineText(0))
        
        if start or attach:
            #detect quotes/args
            self.top.msg("attach=%s" % attach)
            second_quote = attach.find('"', 1)
            if second_quote != -1:
                load = attach[1:second_quote]
                args = attach[second_quote+1:]
                self.top.msg("load1=%s" % load)
            else:
                load = attach
                args = None
                self.top.msg("load2=%s" % load)
            
            #now strip off all but blah.exe
            if load.rfind("\\"):
                self.top.watch = load[load.rfind("\\")+1:]
                self.top.msg("watch1=%s" % self.top.watch)
            else:
                self.top.watch = load
                self.top.msg("watch2=%s" % self.top.watch)
            
            self.top.watch = self.top.watch.lower()
            main = self.top.watch
            self.top.msg("main=%s" % main)
            
        if start and attach:
            #look through the process list, if not found, start it
            for j in range(5):
                for (pid, proc_name) in dbg.enumerate_processes():
                    wx.Yield()
                    self.top.msg("proc_name=%s, looking for:%s" % (proc_name.lower(), main))
                    if proc_name.lower() == main:
                        process_found = True
                        break
                if process_found:
                    self.top.msg("Found target process at %d (0x%04x)" % (pid, pid))
                    attach = pid
                    load   = None
                    args   = None
                    started = True
                    break
                #elif start:
                else:
                    if start:
                        self.top.msg("starting:%s, waiting for:%s secs" % (start, self.top.s_wait))
                        os.startfile(start)
                        time.sleep(self.top.s_wait)
                    else:
                        self.top.err("No start file!")
                        return
                
        # we are attaching to a pid from top select box.
        if not start and not attach and not started:
            if self.selected_pid:
                attach = self.selected_pid
                main   = self.selected_proc.lower()
                load   = None
                args   = None
            else:
                self.top.err("Failed to start and attach to process!  Tried to use PID selection.")
                return
            
        self.top.print_bps = self.top.print_hits.GetValue()

        self.process_stalker = utils.process_stalker(                       \
            attach              = attach,                                   \
            load                = load,                                     \
            args                = args,                                     \
            filter_list         = self.top.filter_list,                     \
            heavy               = self.top.heavy.GetValue(),                \
            ignore_first_chance = self.top.ignore_first_chance.GetValue(),  \
            log                 = self.top.msg,                             \
            main                = main,                                     \
            mysql               = self.top.main_frame.mysql,                \
            host                = self.top.main_frame.host,                 \
            username            = self.top.main_frame.username,             \
            password            = self.top.main_frame.password,             \
            pida_modules        = self.top.pida_modules,                    \
            pydbg               = dbg,                                      \
            print_bps           = self.top.print_bps,                       \
            restore             = self.top.restore_breakpoints.GetValue(),  \
            tag_id              = self.top.stalk_tag["id"],                 \
            target_id           = self.top.stalk_tag["target_id"],          \
            depth               = self.top.coverage_depth.GetSelection(),   \
            top                 = self.top,                                 \
            on_crash            = on_crash
        )

        self.top.attach_detach.SetLabel("Stop Stalking")
        try:
            self.process_stalker.stalk()
        except:
            self.top.err("Exception thrown in stalk!")
            #self.top.err("Trying to terminate process.")
            #self.process_stalker.detach = False
            #self.process_stalker.export = False
            #try:
            #    dbg.terminate_process()
            #except:
            #    self.top.err("Failed to terminate")
                  
        # reset state after stalking is finished.
        self.top.attach_detach.SetLabel("Start Stalking")
        self.process_stalker = None


    ####################################################################################################################
    def on_retrieve_list (self, event):
        pydbg = self.top.main_frame.pydbg

        self.DeleteAllItems()

        idx = 0
        for (pid, proc) in pydbg.enumerate_processes():
            # ignore system processes.
            if pid < 10:
                continue

            self.InsertStringItem(idx, "")
            self.SetStringItem(idx, 0, "%d" % pid)
            self.SetStringItem(idx, 1, proc)
            idx += 1


    ####################################################################################################################
    def on_select (self, event):
        '''
        '''

        self.selected_pid  = int(self.GetItem(event.m_itemIndex, 0).GetText())
        self.selected_proc =     self.GetItem(event.m_itemIndex, 1).GetText()