#
# filefuzzer_dialog
# Copyright (C) 2007 Jared DeMott
#
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

'''
@author:       Jared DeMott
@license:      GNU General Public License 2.0 or later
@contact:      jdemott@vdalabs.com
@organization: VDA Labs, LLC.
'''

import wx
import sys

sys.path.append("..")

import utils

class filefuzzer_dialog(wx.Dialog):
    def __init__(self, *args, **kwds):
        self.parent = kwds["parent"]
        
        # begin wxGlade: fuzzer_connect_dialog.__init__
        kwds["style"] = wx.DEFAULT_DIALOG_STYLE
        wx.Dialog.__init__(self, *args, **kwds)
        
        self.fuzzer_logo = wx.StaticBitmap(self, -1, wx.Bitmap(self.parent.cwd + "/images/fuzzing.bmp", wx.BITMAP_TYPE_ANY))
        self.application_static = wx.StaticText(self, -1, "Application to Fuzz:")
        self.application = wx.TextCtrl(self, -1, "acrord32.exe")
        self.files_dir_static = wx.StaticText(self, -1, "Files Directory:")
        self.files_dir = wx.TextCtrl(self, -1, "NULL")
        self.seedfile_static = wx.StaticText(self, -1, "Seed File:")
        self.seedfile = wx.TextCtrl(self, -1, "NULL")
        self.wait_static = wx.StaticText(self, -1, "Seconds to wait/try:")
        self.wait = wx.TextCtrl(self, -1, "1")
        self.dump_dir_static = wx.StaticText(self, -1, "Exception Dump Dir:")
        self.dump_dir = wx.TextCtrl(self, -1, "NULL")
        self.gpf_id_static = wx.StaticText(self, -1, "GPF_ID:")
        self.gpf_id = wx.TextCtrl(self, -1, "0")
        self.connect = wx.Button(self, -1, "FUZZ")

        self.__set_properties()
        self.__do_layout()

        self.Bind(wx.EVT_BUTTON, self.on_listen, self.connect)
        # end wxGlade

    def __set_properties(self):
        # begin wxGlade: Fuzzer_connect_dialog.__set_properties
        self.SetTitle("EFS File Fuzzer")
        self.application_static.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, "MS Shell Dlg 2"))
        self.application.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, "MS Shell Dlg 2"))
        self.files_dir_static.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, "MS Shell Dlg 2"))
        self.files_dir.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, "MS Shell Dlg 2"))
        self.seedfile_static.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, "MS Shell Dlg 2"))
        self.seedfile.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, "MS Shell Dlg 2"))
        self.wait_static.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, "MS Shell Dlg 2"))
        self.wait.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, "MS Shell Dlg 2"))
        self.dump_dir_static.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, "MS Shell Dlg 2"))
        self.dump_dir.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, "MS Shell Dlg 2"))
        self.gpf_id_static.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, "MS Shell Dlg 2"))
        self.gpf_id.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, "MS Shell Dlg 2"))
        self.connect.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, "MS Shell Dlg 2"))
        self.connect.SetDefault()
        # end wxGlade

    def __do_layout(self):
        # begin wxGlade: filefuzzer_dialog.__do_layout
        sizer_7_copy = wx.BoxSizer(wx.HORIZONTAL)
        sizer_6_copy = wx.BoxSizer(wx.VERTICAL)
        fuzzer_options = wx.GridSizer(2, 2, 0, 0)
        sizer_7_copy.Add(self.fuzzer_logo, 0, wx.ADJUST_MINSIZE, 0)
        sizer_7_copy.Add((10, 20), 0, wx.ADJUST_MINSIZE, 0)
        fuzzer_options.Add(self.application_static, 0, wx.ADJUST_MINSIZE, 0)
        fuzzer_options.Add(self.application, 1, wx.EXPAND|wx.ADJUST_MINSIZE, 0)
        fuzzer_options.Add(self.files_dir_static, 0, wx.ADJUST_MINSIZE, 0)
        fuzzer_options.Add(self.files_dir, 1, wx.EXPAND|wx.ADJUST_MINSIZE, 0)
        fuzzer_options.Add(self.seedfile_static, 0, wx.ADJUST_MINSIZE, 0)
        fuzzer_options.Add(self.seedfile, 1, wx.EXPAND|wx.ADJUST_MINSIZE, 0)
        fuzzer_options.Add(self.wait_static, 0, wx.ADJUST_MINSIZE, 0)
        fuzzer_options.Add(self.wait, 1, wx.EXPAND|wx.ADJUST_MINSIZE, 0)
        fuzzer_options.Add(self.dump_dir_static, 0, wx.ADJUST_MINSIZE, 0)
        fuzzer_options.Add(self.dump_dir, 1, wx.EXPAND|wx.ADJUST_MINSIZE, 0)
        fuzzer_options.Add(self.gpf_id_static, 0, wx.ADJUST_MINSIZE, 0)
        fuzzer_options.Add(self.gpf_id, 1, wx.EXPAND|wx.ADJUST_MINSIZE, 0)
        sizer_6_copy.Add(fuzzer_options, 0, wx.EXPAND, 0)
        sizer_6_copy.Add(self.connect, 0, wx.EXPAND|wx.ADJUST_MINSIZE, 0)
        sizer_7_copy.Add(sizer_6_copy, 1, wx.EXPAND, 0)
        self.SetAutoLayout(True)
        self.SetSizer(sizer_7_copy)
        sizer_7_copy.Fit(self)
        sizer_7_copy.SetSizeHints(self)
        self.Layout()
        # end wxGlade

    def on_listen(self, event): # wxGlade: filefuzzer_dialog.<event_handler>
        try:
            application = self.application.GetLineText(0)
            files_dir = int(self.files_dir.GetLineText(0))
            wait = float(self.wait.GetLineText(0))
            dump_dir = self.dump_dir.GetLineText(0)
            gpf_id = int(self.gpf_id.GetLineText(0))
        except:
            self.parent.status_bar.SetStatusText("Invalid data")
            self.Destroy()
            return

        try:
            self.parent.fuzzer = utils.filefuzzer(application, files_dir, seedfile, wait, dump_dir, gpf_id, parent=self.parent)
        except:
            self.parent.status_bar.SetStatusText("Failed to start filefuzzer.")
            self.Destroy()
            return
            
        self.parent.status_bar.SetStatusText("Fuzzing %s" % application)
        self.Destroy()
