#
# fuzzer_connect_dialog
# Copyright (C) 2006 Jared DeMott
#
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

'''
@author:       Jared DeMott
@license:      GNU General Public License 2.0 or later
@contact:      jdemott@appliedsec.com
@organization: Applied Security, Inc.
'''

import wx
import sys

sys.path.append("..")

import utils

class fuzzer_connect_dialog(wx.Dialog):
    def __init__(self, *args, **kwds):
        self.parent = kwds["parent"]
        
        # begin wxGlade: fuzzer_connect_dialog.__init__
        kwds["style"] = wx.DEFAULT_DIALOG_STYLE
        wx.Dialog.__init__(self, *args, **kwds)
        
        self.fuzzer_logo = wx.StaticBitmap(self, -1, wx.Bitmap(self.parent.cwd + "/images/fuzzing.bmp", wx.BITMAP_TYPE_ANY))
        self.host_static = wx.StaticText(self, -1, "Host:")
        self.host = wx.TextCtrl(self, -1, "0.0.0.0")
        self.port_static = wx.StaticText(self, -1, "Port:")
        self.port = wx.TextCtrl(self, -1, "31338")
        self.wait_static = wx.StaticText(self, -1, "General Wait:")
        self.wait = wx.TextCtrl(self, -1, "0.8")
        self.dump_dir_static = wx.StaticText(self, -1, "Dump Directory:")
        self.dump_dir = wx.TextCtrl(self, -1, "NULL")
        self.gpf_id_static = wx.StaticText(self, -1, "GPF_ID:")
        self.gpf_id = wx.TextCtrl(self, -1, "0")
        self.connect = wx.Button(self, -1, "Listen")

        self.__set_properties()
        self.__do_layout()

        self.Bind(wx.EVT_BUTTON, self.on_listen, self.connect)
        # end wxGlade

    def __set_properties(self):
        # begin wxGlade: Fuzzer_connect_dialog.__set_properties
        self.SetTitle("Listen for Fuzzer Command & Control")
        self.host_static.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, "MS Shell Dlg 2"))
        self.host.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, "MS Shell Dlg 2"))
        self.port_static.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, "MS Shell Dlg 2"))
        self.port.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, "MS Shell Dlg 2"))
        self.wait_static.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, "MS Shell Dlg 2"))
        self.wait.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, "MS Shell Dlg 2"))
        self.dump_dir_static.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, "MS Shell Dlg 2"))
        self.dump_dir.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, "MS Shell Dlg 2"))
        self.gpf_id_static.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, "MS Shell Dlg 2"))
        self.gpf_id.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, "MS Shell Dlg 2"))
        self.connect.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, "MS Shell Dlg 2"))
        self.connect.SetDefault()
        # end wxGlade

    def __do_layout(self):
        # begin wxGlade: fuzzer_connect_dialog.__do_layout
        sizer_7_copy = wx.BoxSizer(wx.HORIZONTAL)
        sizer_6_copy = wx.BoxSizer(wx.VERTICAL)
        fuzzer_options = wx.GridSizer(2, 2, 0, 0)
        sizer_7_copy.Add(self.fuzzer_logo, 0, wx.ADJUST_MINSIZE, 0)
        sizer_7_copy.Add((10, 20), 0, wx.ADJUST_MINSIZE, 0)
        fuzzer_options.Add(self.host_static, 0, wx.ADJUST_MINSIZE, 0)
        fuzzer_options.Add(self.host, 1, wx.EXPAND|wx.ADJUST_MINSIZE, 0)
        fuzzer_options.Add(self.port_static, 0, wx.ADJUST_MINSIZE, 0)
        fuzzer_options.Add(self.port, 1, wx.EXPAND|wx.ADJUST_MINSIZE, 0)
        fuzzer_options.Add(self.wait_static, 0, wx.ADJUST_MINSIZE, 0)
        fuzzer_options.Add(self.wait, 1, wx.EXPAND|wx.ADJUST_MINSIZE, 0)
        fuzzer_options.Add(self.dump_dir_static, 0, wx.ADJUST_MINSIZE, 0)
        fuzzer_options.Add(self.dump_dir, 1, wx.EXPAND|wx.ADJUST_MINSIZE, 0)
        fuzzer_options.Add(self.gpf_id_static, 0, wx.ADJUST_MINSIZE, 0)
        fuzzer_options.Add(self.gpf_id, 1, wx.EXPAND|wx.ADJUST_MINSIZE, 0)
        sizer_6_copy.Add(fuzzer_options, 0, wx.EXPAND, 0)
        sizer_6_copy.Add(self.connect, 0, wx.EXPAND|wx.ADJUST_MINSIZE, 0)
        sizer_7_copy.Add(sizer_6_copy, 1, wx.EXPAND, 0)
        self.SetAutoLayout(True)
        self.SetSizer(sizer_7_copy)
        sizer_7_copy.Fit(self)
        sizer_7_copy.SetSizeHints(self)
        self.Layout()
        # end wxGlade

    def on_listen(self, event): # wxGlade: fuzzer_connect_dialog.<event_handler>
        try:
            host = self.host.GetLineText(0)
            port = int(self.port.GetLineText(0))
            wait = float(self.wait.GetLineText(0))
            dump_dir = self.dump_dir.GetLineText(0)
            gpf_id = int(self.gpf_id.GetLineText(0))
        except:
            self.parent.status_bar.SetStatusText("Invalid hostname/port combination")
            self.Destroy()
            return

        try:
            self.parent.fuzzer = utils.fuzzer_connector(host, port, wait, dump_dir, gpf_id, parent=self.parent)
        except:
            self.parent.status_bar.SetStatusText("Failed to Listen.")
            self.Destroy()
            return
            
        self.parent.status_bar.SetStatusText("Listening on %s." % host)
        self.Destroy()
