#
# Fuzzer Connector
# Copyright (C) 2006 Jared DeMott
#
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# Note: The goal of this connector is to allow a fuzzer to remotely start and stop the stalking process.
#       This should allow the fuzzer to more intelligently fuzz.  My goal is to use a GA fuzzer that
#       learns my using metrics like Code Coverage.
#

'''
@author:       Jared DeMott
@license:      GNU General Public License 2.0 or later
@contact:      jdemott@appliedsec.com
@organization: Applied Security, Inc.
'''

import socket
import thread
import sys
import time
import MySQLdb

class fuzzer_connector:
    '''
    This class provides an abstracted interface for communicating with a fuzzer.
    The fuzzer will be able to start and stop efs and recieve metrics such
    as code coverage.
    '''

    ####################################################################################################################
    def __init__ (self, host="0.0.0.0", port=31338, wait=0.8, dump_dir="NULL", gpf_id=0, *args, **kwds):
        self.parent = kwds["parent"]
        '''
        The init will start the infinite accept loop
        '''
        
        self.sock             = None
        self.log = lambda x: None
        
        self.parent.modules['efs'].dump_dir = dump_dir
        self.parent.modules['efs'].gpf_id = gpf_id
        
        try:
            thread.start_new_thread(fuzzer_connector.accept_fuzzer_control,(self, host, port, wait))
        except:
            self.parent.modules['efs'].err("Thread error.")
            
    ####################################################################################################################
    def connect(self, wait):
        '''
        (re)connect to the mysql db
        '''
        
        i=0
        while 1:
            time.sleep(wait/2)
            try:
                self.parent.mysql = MySQLdb.connect(host=self.parent.host, user=self.parent.username, passwd=self.parent.password, db="paimei")
                break
            except MySQLdb.OperationalError, err:
                i = i + 1
                if i > 20:
                    self.parent.modules['efs'].err("Failed reconnecting to MySQL server: %s" % err[1])
                    raise Exception
        
        
    ####################################################################################################################
    def accept_fuzzer_control (self, host, port, wait):
        '''
        Accept the command (from the fuzzer) to start or stop stalking.
        '''
        last_target_name =""
        first_time = 1
        err_count = 0
        self.parent.modules['efs'].generation = 0
        self.parent.modules['efs'].pool = 0
        self.parent.modules['efs'].session = 0
        self.parent.modules['efs'].crash_number = 0
        
        #setup networking
        try:
            self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            self.sock.bind((host, port))
            self.sock.listen(5)
            self.parent.modules['efs'].msg("Fuzzer Listener Ready.")
            (clientsocket,address) = self.sock.accept()
        except:
            self.parent.modules['efs'].err("socket, bind, or accept failed")    
        
        detach_after_stalk = self.parent.modules['efs'].after_stalk.GetSelection()
        
        #Now we now the total possible hits
        if self.parent.modules['efs'].coverage_depth.GetSelection() == 0:
            total = self.parent.modules['efs'].function_count
        else:
            total = self.parent.modules['efs'].basic_block_count
            
        #create and select a tag for stalking
        target_name = "GPF"
        tag_name = "gpf_stalk_tag"
        self.parent.modules['efs'].targets.on_remote_add_target(target_name)
        self.parent.modules['efs'].targets.on_remote_add_tag(tag_name, target_name)
        self.parent.modules['efs'].targets.on_remote_use_for_stalk(tag_name, target_name)
            
        #loop forever
        while True:
            #grab first cmd -- should be all in one pkt. should be of the form target:tag
            # -->
            try:
                cmd1 = clientsocket.recv(20)
            except:
                self.parent.modules['efs'].err("first recv failed")
                break
            try:
                index = cmd1.find(":")
                target_name = cmd1[:index]
                tag_name = cmd1[index+1:]
                if tag_name == '':
                    try:
                        clientsocket.send("blank tags are invalid")
                        self.parent.modules['efs'].err("bad target:tag parse error. cmd1=%s" % cmd1)
                    except:
                        break
                    continue
            except:
                try:
                    clientsocket.send("!!fuzzer protocol error 1!!")
                    self.parent.modules['efs'].err("target:tag parse error. cmd1=%s" % cmd1)
                except:
                    pass
                break

            #This is for when we're done
            try:
                if target_name == "Done" and tag_name == "OK ":
                    self.parent.modules['efs'].msg("Done. Be Blessed.")
                    self.parent.modules['efs'].targets.on_retrieve_targets(None)
                    break
                elif target_name == "Done" and tag_name == "ERR":
                    self.parent.modules['efs'].err("Stopped becuase of errors")
                    self.parent.modules['efs'].targets.on_retrieve_targets(None)
                    break
            except:
                break
            
            #parse further to get generation + pool
            try:
                space = target_name.index(" ")
                try:
                    underscore = target_name.index("_")
                    self.parent.modules['efs'].generation = int(target_name[space+1:underscore])
                    self.parent.modules['efs'].pool = int(target_name[underscore+1:])
                    self.parent.modules['efs'].session = int(tag_name)
                except:
                    self.parent.modules['efs'].generation = int(target_name[space+1:])
                    self.parent.modules['efs'].pool = 0
                    self.parent.modules['efs'].session = int(tag_name)
            except:
                self.parent.modules['efs'].err("bad target_name")
                break
                
            #start stalking
            if self.parent.modules['efs'].attach_detach.GetLabel() == "Start Stalking":
                try:
                    thread.start_new_thread(self.parent.modules['efs'].process_list.on_attach_detach, (None,))
                except:
                    clientsocket.send("!!failed to start stalking!!")
                    self.parent.modules['efs'].err("failed to start stalking")
                    break
                #(0) - (Wait for startup, loading in of .dll, setting breakpoints, etc.)
                time.sleep(self.parent.modules['efs'].s_wait)
                                
            #Rather we started anew or contined, we need to send this.
            # <--
            try:
                clientsocket.send("Stalking\r\n")
            except:
                self.parent.modules['efs'].err("failed to send 'Stalking'")
                break
            
            #
            #Session play takes place here
            #
            
            #recieve 3 bytes. "aOK", or "err"
            # -->
            try:
                cmd2 = clientsocket.recv(4)
            except:
                self.parent.modules['efs'].err("failed to recieve one or all cc/stop cmd")
                break
            
            #number 1, waiting for the breakpoints to finish being hit    
            time.sleep(wait)
            
            if cmd2 == "err":
                self.connect(wait/2)
                if err_count > 1:
                    self.parent.modules['efs'].err("Change to 'terminate' and restart from previous session")
                    was = self.parent.modules['efs'].after_stalk.GetSelection()
                    self.parent.modules['efs'].after_stalk.SetSelection(2)
                    if self.parent.modules['efs'].attach_detach.GetLabel() == "Start Stalking":
                        self.parent.modules['efs'].process_list.on_attach_detach(None)
                        time.sleep(wait/2)
                    self.parent.modules['efs'].process_list.on_attach_detach(None)
                    self.parent.modules['efs'].attach_detach.SetLabel("Start Stalking")
                    self.parent.modules['efs'].after_stalk.SetSelection(was)
                    time.sleep(wait/2)
                    err_count = err_count + 1
                    continue
                elif self.parent.modules['efs'].attach_detach.GetLabel() == "Start Stalking":
                    self.parent.modules['efs'].err("Restart from previous session")
                    err_count = 1
                    continue
                else:
                    self.parent.modules['efs'].err("Change to 'detach' and restart from previous session")
                    was = self.parent.modules['efs'].after_stalk.GetSelection()
                    self.parent.modules['efs'].after_stalk.SetSelection(1)
                    self.parent.modules['efs'].process_list.on_attach_detach(None)
                    self.parent.modules['efs'].attach_detach.SetLabel("Start Stalking")
                    self.parent.modules['efs'].after_stalk.SetSelection(was)
                    time.sleep(wait/2)
                    err_count = 1
                    continue
                
            #We may have already stoped stalking due to a crash    
            if self.parent.modules['efs'].attach_detach.GetLabel() == "Start Stalking":
                if self.parent.modules['efs'].crashed != 1:
                    self.parent.modules['efs'].err("Target Process likely died for some reason other than access violation")
            else:
            #stop stalking
                try:
                    self.parent.modules['efs'].process_list.on_attach_detach(None)
                except:
                    self.parent.modules['efs'].err("failed detaching")
                    break
            
            #number 2, waiting for mysql to finish         
            time.sleep(wait/2)
            
            
            #clear the hit count incase none get exported next time
            self.parent.modules['efs'].hit = 0
            #reset the app_err counter
            err_count = 0
            #reset the global var after each session
            self.parent.modules['efs'].crashed = 0
            self.parent.modules['efs'].crash_number = 0
        
        try:            
            clientsocket.close()    
            self.parent.modules['efs'].msg("Shutting Down the Fuzzer Listener.")
            self.parent.modules['efs'].msg("Run Fuzzer Connect again to restart.")
            self.parent.status_bar.SetStatusText("Disconnected from GPF.")
            thread.exit()
        except:
            pass